const SDL = @import("sdl2");

const Game = @import("../Game.zig");
const Bag = @import("Bag.zig");
const Grid = @import("Grid.zig");
const Piece = @import("Piece.zig");
pub const Renderer = @import("../Renderer.zig");

const color = @import("../color.zig");

const Self = @This();

renderer: *Renderer,
grid_cell_size: i32,
grid_pos_x: i32,
grid_pos_y: i32,

pub fn init(renderer: *Renderer) Self {
    //var wsize = renderer.getOutputSize();

    const grid_cell_size = @divFloor(@min(640, 480), 32);
    const grid_pos_x = @divFloor(640, 2) - (grid_cell_size * @divFloor(Grid.ncolumns, 2));
    const grid_pos_y = @divFloor(480, 2) - (grid_cell_size * @divFloor(Grid.nrows + Grid.buffer, 2));
    return Self{
        .renderer = renderer,
        .grid_pos_x = grid_pos_x,
        .grid_pos_y = grid_pos_y,
        .grid_cell_size = grid_cell_size,
    };
}

pub fn renderBag(self: *Self, game: Game) void {
    _ = self;

    //const pos_x = self.grid_pos_x + ((Grid.ncolumns + 1) * self.grid_cell_size);
    //const pos_y = self.grid_pos_y + (Grid.buffer * self.grid_cell_size);

    // var r: u8 = 0;
    // var g: u8 = 0;
    // var b: u8 = 0;
    // var a: u8 = 255;

    for (game.bag.contents[0..5], 0..) |_, i| {
        if (game.bag.contents[i]) |piece_type| {
            var piece = Piece.init(piece_type);

            for (piece.structure, 0..) |_, y| {
                for (piece.structure[y], 0..) |_, x| {
                    if (piece.structure[y][x]) {
                        //r = piece.color.r;
                        //g = piece.color.g;
                        //b = piece.color.b;
                        //a = piece.color.a;
                    } else {
                        //r = piece.color.r;
                        //g = piece.color.g;
                        //b = piece.color.b;
                        //a = 50;
                    }

                    //const new_x = pos_x + @as(i32, @intCast(x)) * self.grid_cell_size;
                    //const new_y = pos_y + (@as(i32, @intCast(y)) + @as(i32, @intCast(i * piece.structure.len))) * self.grid_cell_size;

                    //self.renderer.setColor(r, g, b, a);
                    //self.renderer.fillRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
                    //self.renderer.setColor(r -| 30, g -| 30, b -| 30, a);
                    //self.renderer.drawRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
                }
            }
        }
    }
}

pub fn renderHeld(self: *Self, game: Game) void {
    _ = self;

    //const pos_x = self.grid_pos_x - (5 * self.grid_cell_size);
    //const pos_y = self.grid_pos_y + (Grid.buffer * self.grid_cell_size);

    // var r: u8 = 0;
    // var g: u8 = 0;
    // var b: u8 = 0;
    // var a: u8 = 255;

    if (game.held) |held_type| {
        var held = Piece.init(held_type);

        for (held.structure, 0..) |_, y| {
            for (held.structure[y], 0..) |_, x| {
                if (held.structure[y][x]) {
                    //r = held.color.r;
                    //g = held.color.g;
                    //b = held.color.b;
                    //a = held.color.a;
                } else {
                    //r = held.color.r;
                    //g = held.color.g;
                    //b = held.color.b;
                    //a = 50;
                }

                //const new_x = pos_x + @as(i32, @intCast(x)) * self.grid_cell_size;
                //const new_y = pos_y + @as(i32, @intCast(y)) * self.grid_cell_size;

                // self.renderer.setColor(r, g, b, a);
                //self.renderer.fillRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
                // self.renderer.setColor(r -| 30, g -| 30, b -| 30, a);
                //self.renderer.drawRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
            }
        }
    }
}

pub fn render(self: *Self, game: Game) void {
    renderGrid(self, game.grid);
    renderPiece(self, game.piece);
    renderPiece(self, game.shadow);
    renderBag(self, game);
    renderHeld(self, game);
}

pub fn renderPiece(self: *Self, piece: Piece) void {
    _ = self;

    //const pos_x = self.grid_pos_x;
    //const pos_y = self.grid_pos_y;

    for (piece.structure, 0..) |_, y| {
        for (piece.structure[y], 0..) |_, x| {
            // We don't want to paint void cells
            if (piece.structure[y][x] == false) {
                continue;
            }

            //var r = piece.color.r;
            //var g = piece.color.g;
            //var b = piece.color.b;
            //var a = piece.color.a;

            //const new_x = pos_x + (@as(i32, @intCast(x)) + piece.col) * self.grid_cell_size;
            //const new_y = pos_y + (@as(i32, @intCast(y)) + piece.row) * self.grid_cell_size;

            //self.renderer.setColor(r, g, b, a);
            //self.renderer.fillRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
            //self.renderer.setColor(r -| 30, g -| 30, b -| 30, a);
            //self.renderer.drawRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
        }
    }
}

pub fn renderGrid(self: *Self, grid: Grid) void {
    _ = self;

    //const pos_x = self.grid_pos_x;
    //const pos_y = self.grid_pos_y;
    //const lg = color.light_grey;

    var visible = grid.cells[Grid.buffer..];
    for (visible, 0..) |_, y| {
        for (visible[y], 0..) |_, x| {
            _ = x;

            //var r = visible[y][x].color.r;
            //var g = visible[y][x].color.g;
            //var b = visible[y][x].color.b;
            //var a = visible[y][x].color.a;

            //const new_x = pos_x + @as(i32, @intCast(x)) * self.grid_cell_size;
            //const new_y = pos_y + (@as(i32, @intCast(y)) + Grid.buffer) * self.grid_cell_size;

            //self.renderer.setColor(r, g, b, a);
            //self.renderer.fillRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
            //self.renderer.setColor(lg.r, lg.g, lg.b, lg.a);
            //self.renderer.drawRectangle(new_x, new_y, self.grid_cell_size, self.grid_cell_size);
        }
    }
}
